/*!
 * @file quantized_vector.cpp
 * @brief Quantized vector of either 1, 2, 4, 8, 16 bit-width
 * @author SAGAMI, Tsuyoshi <sagami@brains.co.jp>
 *
 * $Id:$
 *
 */
#include <cassert>
#include <iostream>
#include <limits>
#include <utility>
#include <string>

#include "quantized_vector.h"
#include "xml_serdes.h"

using namespace std;


namespace /* unnamed */ {
void
pack_shrink(const vector<uint16_t>& vin, vector<uint8_t>& vout, int nbits)
{
	assert(nbits==1 || nbits==2 || nbits==4 || nbits==8);

	const size_t mask = (1u << nbits) - 1;	  // lower `nbits'
	const size_t elems_per_byte = 8 / nbits; // 1byte=8bitを仮定
	const size_t r = vin.size() % elems_per_byte; // あまり
	const size_t n = vin.size() - r;	     // 割り切れる分
	uint8_t v;
	
	size_t iidx = 0;
	for ( ; iidx < n; iidx += elems_per_byte) {
		v = 0;
		for (size_t j = 0; j < elems_per_byte; ++j) {
			v |= ((vin[iidx + j] & mask) << (j*nbits));
		}
		vout.push_back(v);
	}
	if (r == 0)
		return;

	v = 0;
	for (size_t j = 0; j < elems_per_byte; ++j) { // for あまり
		v |= ((vin[iidx + j] & mask) << (j*nbits));
	}
	vout.push_back(v);
}
void
unpack_shrink(const vector<uint8_t>& vin, vector<uint16_t>& vout, int n_bits, size_t n_elems)
{
	assert(n_bits==1 || n_bits==2 || n_bits==4 || n_bits==8);

	const size_t mask = (1u << n_bits) - 1;	  // lower `n_bits'
	const size_t elems_per_byte = 8 / n_bits; // 1byte=8bitを仮定

	for (size_t i=0, imax=vin.size(); i < imax; ++i) {
		uint8_t v = vin[i];
		for (size_t j = 0; j < elems_per_byte; ++j) {
			vout.push_back((v >> (j*n_bits)) & mask);
		}
	}
	vout.resize(n_elems);
}
void
pack16(const vector<uint16_t>& vin, vector<uint8_t>& vout)
{
	// network byte order
	for (size_t i=0, imax=vin.size(); i < imax; ++i) {
		uint16_t v = vin[i];
		
		vout.push_back((v >> 8) & 0xFFU);
		vout.push_back(v & 0xFFU);
	}
}
void
unpack16(const vector<uint8_t>& vin, vector<uint16_t>& vout)
{
	assert(vin.size() % 2 == 0);

	for (size_t i=0, imax=vin.size(); i < imax; i += 2)
		vout.push_back((vin[i] << 8) | vin[i+1]);
}

} /* unnamed namespace */


/* ---------------------------------------------------------------- *
 *    Class QuantizedVector
 * ---------------------------------------------------------------- */

QuantizedVector::QuantizedVector()
	: data_(), 
	  n_values_(2),
	  range_min_(numeric_limits<value_type>::min()),
	  range_max_(numeric_limits<value_type>::max())
{
}
QuantizedVector::QuantizedVector(size_t n_values, value_type range_min, value_type range_max)
	: data_(), 
	  n_values_(n_values),
	  range_min_(range_min),
		range_max_(range_max)
{
}
int
QuantizedVector::bits_necessary_for(size_t n_values)
{
	if (n_values <= 2) {
		return 1;
	} else if (n_values <= 4) {
		return 2;
	} else if (n_values <= 16) {
		return 4;
	} else if (n_values <= 256) {
		return 8;
	} else if (n_values <= 65536) {
		return 16;
	} else 
		return -1;	// error!
}
bool
QuantizedVector::setRange(value_type range_min, value_type range_max)
{
	if (range_min == range_max)
		return false;

	if (range_min > range_max)
		swap(range_min, range_max);
	range_min_ = range_min;
	range_max_ = range_max;

	return true;
}
bool
QuantizedVector::setNumberOfValues(size_t n_values)
{
	if (n_values-1 > numeric_limits<storage_type>::max())
		return false;
	else if (n_values < 2)	// 0, 1 is useless
		return false;

	n_values_ = n_values;
	return true;
}
QuantizedVector::storage_type
QuantizedVector::quantize(value_type v) const
{
	if (v < range_min_)
		v = range_min_;
	else if (range_max_ < v)
		v = range_max_;

	// truncation toward 0 according to ISO/IEC 14882:2003
	// See 4.9 Floating-integral conversions.
	return storage_type((v - range_min_) / step());
}
QuantizedVector::value_type
QuantizedVector::dequantize(storage_type s) const
{
	const uint32_t mask = (1U <<  bits_necessary_for(n_values_)) - 1;
	return range_min_ + (s & mask) * step();
}

void
QuantizedVector::pack_to(std::vector<uint8_t>& packed) const
{
	int n_bits = bits_necessary_for(n_values_);
	switch (n_bits) {
	case 1:			/* FALLTHROUGH */
	case 2:			/* FALLTHROUGH */
	case 4:			/* FALLTHROUGH */
	case 8:			/* FALLTHROUGH */
		packed.resize(0);
		pack_shrink(data_, packed, n_bits);
		return;
	case 16:
		packed.resize(0);
		pack16(data_, packed);
		return;
	default:
		return;
	}
}
void
QuantizedVector::unpack_from(const std::vector<uint8_t>& packed, int n_bits, size_t n_elems)
{
	switch (n_bits) {
	case 1:			/* FALLTHROUGH */
	case 2:			/* FALLTHROUGH */
	case 4:			/* FALLTHROUGH */
	case 8:			/* FALLTHROUGH */
		unpack_shrink(packed, data_, n_bits, n_elems);
		return;
	case 16:
		unpack16(packed, data_);
		return;
	default:
		return;
	}	
}
void
QuantizedVector::resize(size_type new_size, value_type x)
{
	data_.resize(new_size, quantize(x));
}

int
QuantizedVector::writeToXml(std::ostream& os, const string& tag) const
{
	return XmlSerDes::writeToXmlSub(os, *this, tag);
}
int
QuantizedVector::readFromXml(std::istream& is, const string& tag)
{
	return XmlSerDes::readFromXmlSub(is, *this, tag);
}




/*
 * Local Variables:
 * mode: c++
 * c-basic-offset: 8
 * indent-tabs-mode: t
 * End:
 *
 */

