// -*- C++ -*-
/*!
 * @file  EnvRecognition.h
 * @brief EnvRecognition 
 * @date  $Date$ 
 *
 * $Id$ 
 */
#ifndef ENV_RECOGNITION_H
#define ENV_RECOGNITION_H

#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

#include <vector>

#include "EnvRecognitionConfig.h"

#include "SerDesIF.h"
#include "GridMap.h"
#include "Localization.h"

/** 基本クラス
 *
 * RTCの基本となるクラスからの派生クラス
 */
class EnvRecognition  : public RTC::DataFlowComponentBase{
public:
	/** コンストラクタ
	 */
	EnvRecognition(RTC::Manager* manager);

	/** デストラクタ
	 */
	~EnvRecognition();

	// The initialize action (on CREATED->ALIVE transition)
	// formaer rtc_init_entry() 
	/** 初期化時に実行される
	 *
	 * メモリ確保や, GUI用ウインドウの初期化を行う
	 */
	virtual RTC::ReturnCode_t onInitialize();

	// The activated action (Active state entry action)
	// former rtc_active_entry()
	/** Activate時に実行される
	 *
	 * 設定ファイルの読み込み, 内部パラメータの設定, メモリ確保等の初期化処理を行う.
	 * GUI使用設定時は, ウインドウ処理用スレッドの作成も行う.
	 *
	 * @param[in] ec_id 識別用ID
	 * @return 成功時は, RTC::RTC_OKを返す. それ以外の場合は, エラーコードを返す.
	 */
	virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

	// The deactivated action (Active state exit action)
	// former rtc_active_exit()
	/** Deactivate時に実行される.
	 *
	 * 使用済みメモリの解放を行う.
	 * GUI使用設定時は, ウインドウの破棄, ウインドウ処理用スレッドの停止を待つ.
	 *
	 * @param[in] ec_id 識別用ID
	 * @return 成功時は, RTC::RTC_OKを返す. それ以外の場合は, エラーコードを返す.
	 */
	virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

	// The execution action that is invoked periodically
	// former rtc_active_do()
	/** 実行時に周期的に呼び出される
	 *
	 * 実行時に周期的に呼び出され, メインの処理を行う.
	 *
	 * @param[in] ec_id 識別用ID
	 * @return 成功時は, RTC::RTC_OKを返す. それ以外の場合は, エラーコードを返す.
	 */
	virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

	static const char Category[];
	static const char Name[];


protected:
	// Configuration variable declaration
	// <rtc-template block="config_declare">
	
	// </rtc-template>

	// DataInPort declaration
	// <rtc-template block="inport_declare">

	std::vector<RTC::TimedString *> m_in;
	std::vector<RTC::InPort<RTC::TimedString> *> m_inIn;

	// </rtc-template>

	// DataOutPort declaration
	// <rtc-template block="outport_declare">
	RTC::TimedString m_out; ///< 出力ポートで出力されるデータ
	RTC::OutPort<RTC::TimedString, RTC::NullBuffer> m_outOut; ///< 出力ポート

	// </rtc-template>

	// CORBA Port declaration
	// <rtc-template block="corbaport_declare">

	// </rtc-template>

	// Service declaration
	// <rtc-template block="service_declare">

	// </rtc-template>

	// Consumer declaration
	// <rtc-template block="consumer_declare">

	// </rtc-template>

private:

	/** URGからの受信データ (XML) を処理する
	 *
	 * URGから受信したセンサデータを処理する
	 *
	 * @param[in] recv 受信したXML文字列 (コピーコスト削減のため参照渡し)
	 * @return 成功時は, RTC::RTC_OKを返す. それ以外の場合は, エラーコードを返す.
	 */
	RTC::ReturnCode_t ProcessURGData(const std::string &recv);

	/** 統合結果を送信する
	 *
	 * 統合されたセンサデータをXMLに変換して, 次のコンポーネントへ送信する
	 *
	 * @return 成功時は, RTC::RTC_OKを返す. それ以外の場合は, エラーコードを返す.
	 */
	RTC::ReturnCode_t SendResult(void);

	/** 自己位置推定からの受信データ (XML) を処理する
	 *
	 * 自己位置推定から受信したセンサデータを処理する
	 *
	 * @param[in] recv 受信したXML文字列 (コピーコスト削減のため参照渡し)
	 * @return 成功時は, RTC::RTC_OKを返す. それ以外の場合は, エラーコードを返す.
	 */
	RTC::ReturnCode_t ProcessSelfLocationData(const std::string &recv);

	std::string filename; ///< XML設定ファイル名

	EnvRecognitionConfig *Config;     ///< 設定ファイルのデータクラス

	SerDesIF<GridMap>      *Result;     ///< 結果送信データ用クラス
	SerDesIF<GridMap>      *SensorData; ///< センサデータ受信用クラス
	SerDesIF<Localization> *PosData;    ///< オドメトリ受信用クラス
	GridMap work_map;    ///< 作業用マップ

	CFusion fusion;      ///< センサデータ統合用クラス
};


extern "C"
{
	/** 外界認識モジュールの初期化を行う
	 */
  void EnvRecognitionInit(RTC::Manager* manager);
};

#endif // ENV_RECOGNITION_H
